@@ -12,8 +12,8 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 //
-// $URL$
-// $Id$
+// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-4.14.3/AABB_tree/include/CGAL/AABB_face_graph_triangle_primitive.h $
+// $Id: AABB_face_graph_triangle_primitive.h 9c9d4ea 2018-11-02T14:40:53+01:00 Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0+
 //
 //
@@ -30,11 +30,12 @@
 #include <CGAL/AABB_primitive.h>
 #include <CGAL/boost/graph/property_maps.h>
 #include <CGAL/Default.h>
+#include <boost/mpl/if.hpp>
 
 namespace CGAL {
 
 /*!
- * \ingroup PkgAABB_tree
+ * \ingroup PkgAABBTreeRef
  * Primitive type for a facet of a polyhedral surface.
  * It wraps a handle to a facet of a polyhedron to a 3D triangle.
  * The polyhedron from which the primitive is built should not be deleted
@@ -65,24 +66,27 @@ template < class FaceGraph,
            class CacheDatum=Tag_false >
 class AABB_face_graph_triangle_primitive
 #ifndef DOXYGEN_RUNNING
-  : public AABB_primitive<typename boost::graph_traits<FaceGraph>::face_descriptor,
+  : public AABB_primitive<typename boost::mpl::if_<OneFaceGraphPerTree,
+                                                   typename boost::graph_traits<FaceGraph>::face_descriptor,
+                                                   std::pair<typename boost::graph_traits<FaceGraph>::face_descriptor, const FaceGraph*> >::type,
                         Triangle_from_face_descriptor_map<
                           FaceGraph,
                           typename Default::Get<VertexPointPMap,
                                                 typename boost::property_map< FaceGraph,
-                                                                              vertex_point_t>::type >::type>,
+                                                                              vertex_point_t>::const_type >::type>,
                         One_point_from_face_descriptor_map<
                           FaceGraph,
                           typename Default::Get<VertexPointPMap,
                                                 typename boost::property_map< FaceGraph,
-                                                                              vertex_point_t>::type >::type>,
+                                                                              vertex_point_t>::const_type >::type>,
                         OneFaceGraphPerTree,
                         CacheDatum >
 #endif
 {
-  typedef typename Default::Get<VertexPointPMap, typename boost::property_map< FaceGraph, vertex_point_t>::type >::type VertexPointPMap_;
+  typedef typename Default::Get<VertexPointPMap, typename boost::property_map< FaceGraph, vertex_point_t>::const_type >::type VertexPointPMap_;
+  typedef typename boost::graph_traits<FaceGraph>::face_descriptor FD;
+  typedef typename boost::mpl::if_<OneFaceGraphPerTree, FD, std::pair<FD, const FaceGraph*> >::type Id_;
 
-  typedef typename boost::graph_traits<FaceGraph>::face_descriptor Id_;
   typedef Triangle_from_face_descriptor_map<FaceGraph,VertexPointPMap_>  Triangle_property_map;
   typedef One_point_from_face_descriptor_map<FaceGraph,VertexPointPMap_> Point_property_map;
 
@@ -92,6 +96,16 @@ class AABB_face_graph_triangle_primitive
                           OneFaceGraphPerTree,
                           CacheDatum > Base;
 
+  FD make_id(FD fd, const FaceGraph&, Tag_true)
+  {
+    return fd;
+  }
+
+  std::pair<FD, const FaceGraph*> make_id(FD fd, const FaceGraph& fg, Tag_false)
+  {
+    return std::make_pair(fd, &fg);
+  }
+  
 public:
   #ifdef DOXYGEN_RUNNING
   /// \name Types
@@ -105,9 +119,12 @@ public:
   */
   typedef Kernel_traits<Point>::Kernel::Triangle_3 Datum;
   /*!
-  Id type.
+  Id type:
+  - `boost::graph_traits<FaceGraph>::%face_descriptor` if `OneFaceGraphPerTree` is `CGAL::Tag_true`
+  - `std::pair<boost::graph_traits<FaceGraph>::%face_descriptor, FaceGraph>` if `OneFaceGraphPerTree` is `CGAL::Tag_false`
   */
-  typedef boost::graph_traits<FaceGraph>::face_descriptor Id;
+  unspecified_type Id;
+  
   /// @}
 
   /*!
@@ -117,6 +134,7 @@ public:
   #else
   typedef typename Base::Id Id;
   #endif
+  typedef typename boost::graph_traits<FaceGraph>::face_descriptor face_descriptor;
 
   // constructors
   /*!
@@ -127,7 +145,7 @@ public:
   */
   template <class Iterator>
   AABB_face_graph_triangle_primitive(Iterator it, const FaceGraph& graph, VertexPointPMap_ vppm)
-    : Base( Id_(*it),
+    : Base( Id_(make_id(*it, graph, OneFaceGraphPerTree())),
             Triangle_property_map(const_cast<FaceGraph*>(&graph),vppm),
             Point_property_map(const_cast<FaceGraph*>(&graph),vppm) )
   {}
@@ -137,8 +155,8 @@ public:
     If `VertexPointPMap` is the default of the class, an additional constructor
     is available with `vppm` set to `get(vertex_point, graph)`.
   */
-  AABB_face_graph_triangle_primitive(Id id, const FaceGraph& graph, VertexPointPMap_ vppm)
-    : Base( Id_(id),
+  AABB_face_graph_triangle_primitive(face_descriptor fd, const FaceGraph& graph, VertexPointPMap_ vppm)
+    : Base( Id_(make_id(fd, graph, OneFaceGraphPerTree())),
             Triangle_property_map(const_cast<FaceGraph*>(&graph),vppm),
             Point_property_map(const_cast<FaceGraph*>(&graph),vppm) )
   {}
@@ -146,13 +164,13 @@ public:
 #ifndef DOXYGEN_RUNNING
   template <class Iterator>
   AABB_face_graph_triangle_primitive(Iterator it, const FaceGraph& graph)
-    : Base( Id_(*it),
+    : Base( Id_(make_id(*it, graph, OneFaceGraphPerTree())),
             Triangle_property_map(const_cast<FaceGraph*>(&graph)),
             Point_property_map(const_cast<FaceGraph*>(&graph)) )
   {}
 
-  AABB_face_graph_triangle_primitive(Id id, const FaceGraph& graph)
-    : Base( Id_(id),
+  AABB_face_graph_triangle_primitive(face_descriptor fd, const FaceGraph& graph)
+    : Base( Id_(make_id(fd, graph, OneFaceGraphPerTree())),
             Triangle_property_map(const_cast<FaceGraph*>(&graph)),
             Point_property_map(const_cast<FaceGraph*>(&graph)) )
   {}
